function [output_img] = compositeimage(img_to_transform, fixed_img, tform, algorithm)
%COMPOSITEIMAGE Generate a single image from a pair of images and a transformation
%   [OUTPUT_IMG] = COMPOSITEIMAGE(IMG_TO_TRANSFORM, FIXED_IMG, TFORM) transforms IMG_TO_TRANSFORM
%   according to TFORM, and combines the transformed image with FIXED_IMG to create the result.
%
%   input parameters:
%       IMG_TO_TRANSFORM - Image to transform and stitch.
%       FIXED_IMG        - Second image to stitch (without changing it).
%       TFORM            - A projective transformation structure, as returned by cp2tform() or
%                          maketform().
%       ALGORITHM        - Either 'simple', 'overlay' or 'awesome', as explained below.
%
%   output parameters:
%       OUTPUT_IMG - The compisite image.
%
%   method == 'simple'
%   Combines the two images by taking all valid pixels from one of the images, and the rest from the
%   other image.
%
%   method == 'overlay'
%   Creates an overlay image, useful for debugging.
%
%   method == 'awesome'
%   Your own algorithm. Note that this option should produce a visually pleasing composite image 
%   (similar to 'simple, only better), and not output which is useful for debugging such as 
%   'overlay'.
%

switch algorithm
  case 'simple'
    output_img = compositeimagesimple(img_to_transform, fixed_img, tform);
    
  case 'overlay'
    output_img = compositeimageoverlay(img_to_transform, fixed_img, tform);
    
  case 'awesome'
    error('compositeimage(): awesome not implemented yet.');
    
  otherwise
    error('compositeimage(): Invalid algorithm.');
end

end


function [output_img] = compositeimageoverlay(img_to_transform, fixed_img, tform)
%COMPOSITEOVERLAYIMAGE Generate an overlay image from a pair of images and a transformation
%   [OUTPUT_IMG] = COMPOSITEOVERLAYIMAGE(IMG_TO_TRANSFORM, FIXED_IMG, TFORM) transforms 
%   IMG_TO_TRANSFORM according to TFORM, and combines the transformed image with FIXED_IMG to create 
%   an overlay image. In the overlay image, gray marks areas where the two images are similar, while 
%   green/magenta mark differences.
%
%   input parameters:
%       IMG_TO_TRANSFORM - Image to transform and compare.
%       FIXED_IMG        - Second image to compare (without changing it).
%       TFORM            - A projective transformation structure, as returned by cp2tform() or
%                          maketform().
%
%   output parameters:
%       OUTPUT_IMG - The overlay image.
%

% Transform img_to_transform according to tform
[trans_img, xdata, ydata] = imtransform(img_to_transform, tform, 'XYScale', 1);

% Combine the two images into a final output
trans_ref = imref2d(size(trans_img), xdata, ydata);
fixed_ref = imref2d(size(fixed_img));
output_img = imfuse(trans_img, trans_ref, fixed_img, fixed_ref);

end


function [output_img] = compositeimagesimple(img_to_transform, fixed_img, tform)

% Transform the first image
[trans_img, xdata, ydata] = imtransform(img_to_transform, tform, 'XYScale', 1);

% Combine the two images into one

[trans_img_nrows, trans_img_ncols, ~] = size(trans_img);
[fixed_img_nrows, fixed_img_ncols, num_channels] = size(fixed_img);

if (xdata(1) < 0)
  xdisp = round(-xdata(1) + 1);
  
  if (ydata(1) < 0)
    ydisp = round(-ydata(1) + 1);
    imgsizex = max(trans_img_nrows, fixed_img_nrows + ydisp - 1);
    imgsizey = max(trans_img_ncols, fixed_img_ncols + xdisp - 1);
    
    canvas1 = uint8(zeros(imgsizex, imgsizey, 3));
    canvas2 = uint8(zeros(imgsizex, imgsizey, 3));
    
    canvas1(1:(trans_img_nrows), 1:trans_img_ncols, 1:num_channels) = trans_img;
    canvas2(ydisp:(fixed_img_nrows + ydisp - 1), xdisp:(fixed_img_ncols + xdisp - 1), ...
      1:num_channels) = fixed_img;   
  else
    ydisp = round(1 + ydata(1));
    imgsizex = max(trans_img_nrows + ydisp - 1, fixed_img_nrows);
    imgsizey = max(trans_img_ncols, xdisp + fixed_img_ncols - 1);
    
    canvas1 = uint8(zeros(imgsizex, imgsizey, 3));
    canvas2 = uint8(zeros(imgsizex, imgsizey, 3));
    
    canvas1(ydisp:(ydisp + trans_img_nrows - 1), 1:(trans_img_ncols), 1:num_channels) = trans_img;
    canvas2(1:fixed_img_nrows, xdisp:(xdisp + fixed_img_ncols - 1), 1:num_channels) = fixed_img;   
  end
else
  xdisp = round(xdata(1) + 1);
  
  if (ydata(1) < 0)
    ydisp = round(-ydata(1) + 1);
    imgsizex = max(trans_img_nrows, fixed_img_nrows + ydisp - 1);
    imgsizey = max(trans_img_ncols + xdisp - 1, fixed_img_ncols);
    
    canvas1 = uint8(zeros(imgsizex, imgsizey, 3));
    canvas2 = uint8(zeros(imgsizex, imgsizey, 3));
    
    canvas1(1:trans_img_nrows, xdisp:(trans_img_ncols + xdisp - 1), 1:num_channels) = trans_img;
    canvas2(ydisp:(ydisp + fixed_img_nrows - 1), 1:fixed_img_ncols, 1:num_channels)= fixed_img; 
  else
    ydisp = round(1 + ydata(1));
    imgsizex = max(trans_img_nrows + ydisp - 1, fixed_img_nrows);
    imgsizey = max(trans_img_ncols + xdisp - 1, fixed_img_ncols);
    
    canvas1 = uint8(zeros(imgsizex, imgsizey, 3));
    canvas2 = uint8(zeros(imgsizex, imgsizey, 3));
    
    canvas1(ydisp:(trans_img_nrows + ydisp - 1), xdisp:(xdisp + trans_img_ncols - 1), ...
      1:num_channels) = trans_img;
    canvas2(1:fixed_img_nrows, 1:fixed_img_ncols, 1:num_channels) = fixed_img; 
  end
end

canvassub = imsubtract(canvas1, canvas2);
output_img = imadd(canvassub, canvas2);

end
