function [descriptors] = computedescriptors(input_image, features, algorithm)
%COMPUTEDESCRIPTORS Generate descriptors of given features
%   [DESCRIPTORS] = COMPUTEDESCRIPTORS(INPUT_IMAGE, FEATURES, ALGORITHM) takes as input an RGB 
%   image and locations/scales/orientations within that image that represent salient points. The 
%   function returns descriptors for each of the point. Different implementations are supplied, 
%   according to the value of 'algorithm'.
%
%   input parameters:
%       INPUT_IMAGE - PxQx3 matrix representing an image.
%       FEATURES    - 4xF matrix. Each column contains x-location, y-location, scale and 
%                     orientation.
%       ALGORITHM   - Either 'random', 'sift', 'window' or 'awesome', as explained below.
%
%   output parameters:
%       DESCRIPTORS - KxF matrix. K is the size of the descriptor, which will be different for 
%                     different algorithms (e.g., K=128 for SIFT).
%
%   method == 'random'
%   Returns random numbers as a descriptor.
%
%   method == 'sift'
%   The descriptors are calculated according to the sift algorithm. The algorithm returns a 128xF 
%   matrix.
%
%   method == 'window'
%   Return a (k<sup>2</sup>)xF  matrix containing the
%   luminance sampled on a kxk rectangular grid of locations centered at the feature position
%   after the neighborhood has been scaled and rotated according to the 
%   scale and orientation of the feature (e.g, k=7).
%
%   method == 'awesome'
%   Your own algorithm.
%

switch algorithm
  case 'random'
    DESCRIPTOR_SIZE = 128;
    descriptors = rand(DESCRIPTOR_SIZE, size(features, 2));
    
  case 'sift'
    img_gray = rgb2gray(input_image);
    [descriptors] = sift_computedescriptors(img_gray, features);
        
  case 'window'
    error('computedescriptors(): window not implemented yet.');
    
  case 'awesome'
    error('computedescriptors(): awesome not implemented yet.');
    
  otherwise
    error('computedescriptors(): Invalid algorithm.');
end

end

%#ok<*INUSL>