function [err] = computeerror(fixed_image_size, transformed_image_size, ...
    tform_ground_truth, tform_to_check)
%COMPUTEERROR Computes the error of ground truth vs. some transformation.
%   The error is computed as follows:
%   We look at all the ground truth pixels (in the transformed image) that are transformed to
%   locations which overlap with the fixed image. For these pixels, we calculate the L2 distance
%   between ground truth location and tform location (both after performing the transformation). The
%   average of those distances is our measurement.
%
%   input parameters:
%       FIXED_IMAGE_SIZE       - Size of fixed image.
%       TRANSFORMED_IMAGE_SIZE - Size of transformed image.
%       TFORM_GROUND_TRUTH     - The "true" transformation.
%       TFORM_TO_CHECK         - Result of our algorithm.
%
%   output parameters:
%       ERR - Error measurement, as described above.
%

% Create a grid of pixel locations
[grid_x, grid_y] = meshgrid(0.5:1:transformed_image_size(2) - 0.5, ...
                            0.5:1:transformed_image_size(1) - 0.5);

% Transform the locations according to 'tform_ground_truth' and 'tform_to_check'
[ground_truth_x, ground_truth_y] = tformfwd(tform_ground_truth, grid_x(:), grid_y(:));
[check_x, check_y] = tformfwd(tform_to_check, grid_x(:), grid_y(:));

% Find locations which do not overlap with the fixed image. We do not use these when calculating the
% score.
no_overlap_indices = (ground_truth_x < 0) | (ground_truth_x > fixed_image_size(2)) | ...
                     (ground_truth_y < 0) | (ground_truth_y > fixed_image_size(1));
% Remove locations which do not overlap.
ground_truth_x(no_overlap_indices) = [];
ground_truth_y(no_overlap_indices) = [];
check_x(no_overlap_indices) = [];
check_y(no_overlap_indices) = [];

% Calculate error.
err = sum(sqrt((ground_truth_x - check_x) .^ 2 + (ground_truth_y - check_y) .^ 2));
err = err ./ numel(ground_truth_x);

end

