function [transform] = computetransform(features1, features2, correspondences, algorithm, ...
                                        ground_truth_filename)
%COMPUTETRANSFORM Computes homography from given point correspondences.
%   [TRANSFORM] = COMPUTETRANSFORM(FEATURES1, FEATURES2, CORRESPONDENCES, ALGORITHM,
%                                  GROUND_TRUTH_FILENAME) takes as input features and
%   correspondences, and computes the transformation that best explains the correspondences.
%
%   input parameters:
%       FEATURES1             - features of the 1st image, as described in detectfeatures().
%       FEATURES2             - features of the 2nd image, as described in detectfeatures().
%       CORRESPONDENCES       - 2xC matrix of correspondences, as described in findcorrespondences().
%       ALGORITHM             - Either 'groundtruth' or 'cp2tform'.
%       GROUND_TRUTH_FILENAME - Name of the file that stores the ground truth transformation. 
%
%   output parameters:
%       TRANSFORM - A transformation structure.
%
%   method == 'groundtruth'
%   Returns the "true" transformation.  
%
%   method == 'cp2tform'
%   Uses the correspondences to calculate a transformation.
%
%   IMPORTANT NOTE: if you want to test your results, do NOT use the 'groundtruth' option, since it
%   discards all previous results and uses the ground truth transform. Use it only if you want to
%   view the ground truth.
%

switch algorithm
  case 'groundtruth'
    ground_truth = load(ground_truth_filename, 'tform');
    transform = ground_truth.tform;
    
  case 'cp2tform'
    points1 = features1(1:2, correspondences(1, :))';
    points2 = features2(1:2, correspondences(2, :))';
    transform = cp2tform(points1, points2, 'projective');
        
  otherwise
    error('computetransform(): Invalid algorithm.');
end

end

