function [features] = detectfeatures(input_image, max_features, algorithm)
%DETECTFEATURES Generate features corresponding to salient points of an image
%   [FEATURES] = DETECTFEATURES(INPUT_IMAGE, MAX_FEATURES, ALGORITHM) takes as input an RGB image,
%   detects salient points in the image and generates features to represent those points. The
%   function supplies different implementations, according to the value of 'algorithm'.
%
%   input parameters:
%       INPUT_IMAGE  - PxQx3 matrix representing an image.
%       MAX_FEATURES - Number of features to return. 'inf' will return all detected features.
%       ALGORITHM    - Either 'random', 'sift', 'harris' or 'awesome', as explained below.
%
%   output parameters:
%       FEATURES - 4xF matrix. Each column contains x-location, y-location, scale and orientation.
%                  For example, locations(2, 5) is the y coordinate of the 5th salient point.
%                  location(3, 10) is the scale of the 10th point.
%                  F equals the number of detected features or max_features, the smaller of the two.
%
%   method == 'random'
%   Returns random numbers as features. 
%
%   method == 'sift'
%   The features are calculated according to the sift algorithm.
%
%   method == 'harris'
%   Corners, as detected by a harris corner detector, are considered salient points.
%
%   method == 'awesome'
%   Your own algorithm.
%

switch algorithm
  case 'random'
    MAX_RANDOM_FEATURES = 2000;
    num_features = min(MAX_RANDOM_FEATURES, max_features);
    features = [rand(1, num_features) .* size(input_image, 2) ; ...
                rand(1, num_features) .* size(input_image, 1) ; ...
                rand(2, num_features)];
    
  case 'sift'
    img_gray = rgb2gray(input_image);
    features = sift_detectfeatures(img_gray, max_features);
    
  case 'harris'
    error('detectfeatures(): harris not implemented yet.');
    
  case 'awesome'
    error('detectfeatures(): awesome not implemented yet.');
    
  otherwise
    error('detectfeatures(): Invalid algorithm.');
end

end

