function [correspondences] = findcorrespondences(features1, descriptors1, features2, ...
                                                 descriptors2, matches, algorithm)
%FINDCORRESPONDENCES Finds good correspondances that form a good transformation between features.
%   [CORRESPONDENCES] = FINDCORRESPONDENCES(FEATURES1, DESCRIPTORS1, FEATURES2, DESCRIPTORS2, 
%                                           MATCHES, ALGORITHM) computes the most plausible
%   correspondences according to a set of features/descriptors and the matching between them.
%
%   input parameters:
%       FEATURES1             - features of the 1st image, as described in detectfeatures().
%       DESCRIPTORS1          - descriptors of the 1st image, as described in computedescriptors().
%       FEATURES2             - features of the 2nd image, as described in detectfeatures().
%       DESCRIPTORS2          - descriptors of the 2nd image, as described in computedescriptors().
%       MATCHES               - as described in findmatches().
%       ALGORITHM             - Either 'random', 'RANSAC' or 'awesome', as explained below.
%
%   output parameters:
%       CORRESPONDENCES - 2xC matrix, where each column contains an index to FEATURES1/DESCRIPTORS1 
%                         and an index to FEATURES2/DESCRIPTORS2.
%                         C equals the number of detected correspondences.
%
%   method == 'random'
%   Return random correspondences. 
%
%   method == 'RANSAC'
%   Return the best set of inlier feature correspondences found with the RANSAC algorithm,
%   as described in class.
%
%   method == 'awesome'
%   Your own algorithm.
%
                                  
switch algorithm
  case 'random'
    MAX_NUMBER_OF_CORRS = 30;
    number_of_corrs = min(size(matches, 2), MAX_NUMBER_OF_CORRS);
    correspondences = matches(:, randperm(size(matches, 2), number_of_corrs));
    
  case 'RANSAC'
    error('findcorrespondences(): RANSAC not implemented yet.');
    
  case 'awesome'
    error('findcorrespondences(): awesome not implemented yet.');
        
  otherwise
    error('findcorrespondences(): Invalid algorithm.');
end

end


%#ok<*INUSL>



