function [matches] = findmatches(features1, descriptors1, features2, descriptors2, max_matches, ...
                                 algorithm)
%FINDMATCHES Given two sets of features and descriptors, finds matching couples.
%   [MATCHES] = FINDMATCHES(FEATURES1, DESCRIPTORS1, FEATURES2, DESCRIPTORS2, MAX_MATCHES, 
%     ALGORITHM) is used to find matching pair of descriptors, such that we 
%   have high confident that such a pair is indeed a match.
%
%   input parameters:
%       FEATURES1    - 4xF matrix. Each column contains x-location, y-location, scale and 
%                      orientation.
%       DESCRIPTORS1 - KxF matrix. K is the size of the descriptor, which will be different for 
%                      different algorithms (e.g., K=128 for SIFT).
%       FEATURES2    - Second set of features.
%       DESCRIPTORS2 - Second set of descriptors.
%       MAX_MATCHES  - Maximum number of matches to return. 'inf' will return all detected matches.
%       ALGORITHM    - either 'random', 'mutual', 'ratio' or 'awesome', as explained below.
%
%   output parameters:
%       MATCHES - 2xM matrix, where each column contains an index to FEATURES1/DESCRIPTORS1 and an
%                 index to FEATURES2/DESCRIPTORS2. For example, a column of [2 ; 200] means the 2nd
%                 feature in group 1 matches the 200th feature in group 2.
%                 M equals the number of detected matches or max_matches, the smaller of the two.
%
%   method == 'random'
%   Returns random matches. 
%
%   method == 'mutual'
%   Return all matches between features i1 and i2 where 
%   the L2 distance between descriptors i1 and i2 is less than the L2 
%   distance between descriptor i1 and any other in descriptors2 AND 
%   also less than the L2 distance between descriptor i2 and any other 
%   in descriptors1.
%
%   method == 'ratio'
%   Return all matches between features i1 and i2 where
%   the L2 distance between descriptors i1 and i2 is less a constant match_ratio
%   times the L2 distance between descriptor i1 and second best match
%   in descriptors2 (e.g., match_ratio=0.6).
%
%   method == 'awesome'
%   Your own algorithm.
%   

switch algorithm
  case 'random'
    NUMBER_OF_MATCHES = 200;
    num_matches = min(NUMBER_OF_MATCHES, max_matches);
    matches = [randi(size(features1, 2), 1, num_matches) ; ...
               randi(size(features2, 2), 1, num_matches)];
    
  case 'mutual'
    error('findmatches(): mutual not implemented yet.');
    
  case 'ratio'
    error('findmatches(): ratio not implemented yet.');
    
  case 'awesome'
    error('findmatches(): awesome not implemented yet.');
    
  otherwise
    error('findmatches(): Invalid algorithm.');
end

end

%#ok<*INUSD,*INUSL>
