function [] = runme(force_overwrite)
%RUNME Entry point for assignment 2
%   [] = RUNME(FORCE_OVERWRITE) executes the various steps needed to stitch a pair of images. 
%
%   input parameters:
%       FORCE_OVERWRITE - Optional. If 'true', all outputs will be generated, even if they already 
%                         exist. Default is 'false'.
%

if ~exist('sift.m', 'file')
    addpath('sift');
end

% By default we do not overwrite results
if (nargin < 1)
  force_overwrite = false;
end

% Edit this cell array to decide on the specific configurations you would like to execute.
% Each cell is a cell array on its own. The first member of that array is the name of the input
% example (i.e. filename without the '_0?.jpg' suffix). 'all' is a reserved word that causes all
% inputs to run with the specified parameters. The rest of the cells specify the algorithm for each
% of the steps.

run_configurations = { ...
   % {'all', 'sift', 'sift', 'ratio', 'RANSAC', 'cp2tform', 'simple'}, ...
   % {'test01', 'harris', 'window', 'ratio', 'RANSAC', 'cp2tform', 'simple'}, ...
   % {'test01', 'harris', 'sift', 'ratio', 'RANSAC', 'cp2tform', 'simple'}, ...
   % {'test01', 'sift', 'window', 'ratio', 'RANSAC', 'cp2tform', 'simple'}, ...
   % {'test01', 'sift', 'sift', 'ratio', 'RANSAC', 'cp2tform', 'simple'}, ...
   {'all', 'random', 'random', 'random', 'random', 'groundtruth', 'simple'}
};

% Will hold the error for each input sample
stitching_errors = [];

% Iterate all running configurations
for configuration = run_configurations
  basename = configuration{1}{1};
  
  if (strcmpi('all', basename))
    % Iterate over all files and run with the given parameters
    
    % Get list of all filenames in input directory
    filenames = dir('../input');

    % Process all filenames in input directory
    for i = 1:size(filenames)
      [~, basename, ext] = fileparts(filenames(i).name);
      
      % Iterate all .jpg files
      if (~filenames(i).isdir && strcmpi(ext, '.jpg'))
        % Determine input and output filenames
        
        % we assume the image number within a test set is always 2 digits
        image_number = str2double(basename(end - 1:end));
        % we handle each test set only once
        if (image_number ~= 1)
          continue;
        end
        
        base_filename = basename(1:end - 3);
        
        % Do the heavy lifting
        err = processtestcase(base_filename, configuration{1}{2:end}, force_overwrite);
        stitching_errors(end + 1) = err;
      end
    end
  else
    % Do the heavy lifting
    err = processtestcase(basename, configuration{1}{2:end}, force_overwrite);
    stitching_errors(end + 1) = err;
  end
end

% Plot cdf of errors and save to file.
% Note that only the errors computed in the last run will be plotted, thus in order to get a CDF of
% all your inputs, use runme(true) to force recalculation of all results.
if (~isempty(stitching_errors) && ~all(isnan(stitching_errors)))
  h = figure('Visible', 'Off');
  cdfplot(stitching_errors);
  print(h, '-djpeg', '-r300', '../output/cdf.jpg');
  close(h);
end

end


% processtestcase() implements the various steps, as described on the assignment website.
function [err] = processtestcase(base_filename, a_method, b_method, c_method, d_method, ...
                                 e_method, f_method, force_overwrite)
  err = nan;

  % Generate file names
  base_filename_full = strjoin({base_filename, a_method, b_method, c_method, d_method, e_method, ...
                                f_method}, '_');
  input_filename1 = ['../input/', base_filename, '_01.jpg'];
  input_filename2 = ['../input/', base_filename, '_02.jpg'];
  output_filename = ['../output/', base_filename_full, '.jpg'];
  overlay_filename = ['../overlay/', base_filename_full, '.jpg'];
  matching_filename = ['../matching/', base_filename_full, '.jpg'];
  features_filename1 = ['../features/', base_filename_full, '_01.jpg'];
  features_filename2 = ['../features/', base_filename_full, '_02.jpg'];
  corrs_filename = ['../correspondence/', base_filename_full, '.jpg'];
  ground_truth_filename = ['../ground_truth/', base_filename, '.mat'];

  % make sure files exist
  assert(2 == exist(input_filename1, 'file'), ['Missing file detected: ' input_filename1]);
  assert(2 == exist(input_filename2, 'file'), ['Missing file detected: ' input_filename2]);

  % We proceed if image does not exist or if 'force_rewrite' equals true.
  if (~force_overwrite && exist(output_filename, 'file') && exist(matching_filename, 'file'))
    disp(['[-] Skipping ' base_filename]);
    return;
  end

  disp(['[+] Processing ' base_filename]);
    
  % Read input images
  input_image1 = imread(input_filename1);
  input_image2 = imread(input_filename2);      

  % Step A: Feature detection
  features1 = detectfeatures(input_image1, inf, a_method);
  features2 = detectfeatures(input_image2, inf, a_method);

  % Visualize the features
  disp(['    [+] Creating ' features_filename1]);
  writefeaturesimage(features_filename1, input_image1, features1);
  disp(['    [+] Creating ' features_filename2]);
  writefeaturesimage(features_filename2, input_image2, features2);

  % Step B: Feature description
  descriptors1 = computedescriptors(input_image1, features1, b_method);
  descriptors2 = computedescriptors(input_image2, features2, b_method);

  % Step C: Feature matching
  [matches] = findmatches(features1, descriptors1, features2, descriptors2, inf, c_method);
  
  % Visualize the matching
  disp(['    [+] Creating ' matching_filename]);
  writematchingimage(matching_filename, input_image1, input_image2, features1, features2, ...
                     matches);
                   
  % Step D: Feature correspondence
  [correspondences] = findcorrespondences(features1, descriptors1, features2, descriptors2, ...
                                          matches, d_method);
  % Visualize the correspondences
  disp(['    [+] Creating ' corrs_filename]);
  writematchingimage(corrs_filename, input_image1, input_image2, features1, features2, ...
                     correspondences, 1);
                   
  % Step E: Homography estimation
  [tform] = computetransform(features1, features2, correspondences, e_method, ...
                             ground_truth_filename);
  
  if (isempty(tform))
    disp(['[-] Unable to extract transform for ' base_filename]);
    return;
  end

  % Step F: create panorama
  output_image = compositeimage(input_image1, input_image2, tform, f_method);
  output_image_overlay = compositeimage(input_image1, input_image2, tform, 'overlay');

  % Create and save output images
  disp(['    [+] Creating ' output_filename]);
  imwrite(output_image, output_filename);
  disp(['    [+] Creating ' overlay_filename]);
  imwrite(output_image_overlay, overlay_filename);

  % Compute score (vs. ground truth)
  if (2 == exist(ground_truth_filename, 'file'))
    ground_truth = load(ground_truth_filename, 'tform');
    err = computeerror(size(input_image2), size(input_image1), ground_truth.tform, tform);
  else
    disp(['[-] Ground truth not found for ' base_filename]);
    err = nan;
  end
end


% Create an image to visualize the features
function [] = writefeaturesimage(filename, img, features)
  h = figure('Visible', 'Off');
  imagesc(img);
  colormap('gray');
  axis('image');
  hold('on');
  hh = plotsiftframe(features, 'Style', 'Arrow');
  set(hh, 'LineWidth', 1);
  print(h, '-djpeg', '-r300', filename);
  close(h);
end

% Create an image to visualize the matching
function [] = writematchingimage(filename, img1, img2, features1, features2, matches, ...
                                 display_fraction)

% display_fraction indicates how many matches we wish to visualize. 1 indicates all samples, but
% usually results in a crowded output image. Default is 0.15.
if (nargin < 7)
  display_fraction = 0.15;
end

% select a random subset of matches to display
num_matches = size(matches, 2);
num_samples = floor(num_matches * display_fraction);
matches = matches(:, randperm(num_matches, num_samples));

% plot matches and save to file
h = figure('Visible', 'Off');
plotmatches(im2double(img1), im2double(img2), features1, features2, matches);
print(h, '-djpeg', '-r300', filename);
close(h);

end


%#ok<*AGROW>