function [tform] = runme_generategroundtruth(base_filename)
%RUNME_GENERATEGROUNDTRUTH Used to generate ground truth transformations.
%   [TFORM] = RUNME_GENERATEGROUNDTRUTH(BASE_FILENAME) generates the ground truth for input 
%   BASE_FILENAME. The ground truth is created via an interactive GUI that allows the user to
%   manually annotate matching points in the two images.
%
%   input parameters:
%       BASE_FILENAME - Name of input sample. For example, 'test01'.
%
%   output parameters:
%       TFORM - The "true" transform.
%
%   Note: do not run this with the given inputs, as this will overwrite the provided ground
%   truth. This function is intended to be used on your own examples.
%

% Generate file names
moving_img_filename = ['../input/', base_filename, '_01.jpg'];
fixed_img_filename = ['../input/', base_filename, '_02.jpg'];
output_filename = ['../ground_truth/', base_filename, '.mat'];

% Read the images
fixed_img = imread(fixed_img_filename);
moving_img = imread(moving_img_filename);

% Let the user interactively select matching points
[moving_points, fixed_points] = cpselect(moving_img, fixed_img, 'Wait', true);

% Create a transformation from the given matches
tform = cp2tform(moving_points, fixed_points, 'projective');

% Create overlay image
[output_img_diffs] = compositeimage(moving_img, fixed_img, tform, 'overlay');

% Display the images. A good transformation will result in a mostly gray image, while green-magenta
% colors indicate areas where pixel values are different.
% Notice that if the 2 images are very different (e.g., due to illumination difference), no gray
% will be shown, even for a perfect transformation.
imshow(output_img_diffs);

% Save the transformation matrix for later use
save(output_filename, 'tform');

end